function [ X_fil, X_filvar, Y_pre_error, Y_pre_var, X_pre_var, loglik] =...
    Kalman(data, F, V, H, W, InitX_mean, InitX_var, eps_var, eta_var)
%Kalman performs Kalman filter for the state space model in
%the following form:
%
%   State equation:        X(t) = F*X(t-1) + V*eps
%   Measurement equation:  Y(t) = H*X(t) + W*eta
%      X --->   nX x 1      Y --->   nY x 1
%    eps ---> nEps x 1    eta ---> nEta x 1
%
% -------------------------------------------------------------------------
% USAGE: [ output_args ] = Kalman(data, F, H, W, Init_mean, Init_var)
% where: 
%       data = (T x nY)  data matrix Y
%          F = (nX x nX) matrix in the state equation
%          V = (nX x nEps) shock loading in the state equation
%          H = (nY x nX) matrix in the measurement equation
%          W = (nY x nEta) shock loading in measurement equation
% InitX_mean = (nX x 1)  initial value for the state vector
%  InitX_var = (nX x nX) initial value for the state variance
%    eps_var = (nEps x nEps) state error variance
%    eta_var = (nEta x nEta) measurement error variance
% -------------------------------------------------------------------------
% RETURNS: 
%       X_fil  = (nx x nt) vector of filtered means
%    X_filvar  = (nx x nx x T) array of filtered variances
% -------------------------------------------------------------------------
% written by:
% Ben Wang
% Dept of Economics
% Macquarie University
% ben.wang@mq.edu.au
% -------------------------------------------------------------------------
% VERSION HISTORY:
% v1.0: 04/03/2011  Setup Kalman filter algorithm.

%% INITIALISATION
%Determine data length:
T = length(data(:,1));
nY = length(data(1,:));
nX = length(InitX_mean(:,1));
nEta = length(W(1,:));
nEps = length(V(1,:));
%Check Dimentions:
if (nY~=size(H,1) || nY~=size(W,1))
	error('# of obervable variable specified in the data is different from matrix H and W');
elseif (nX~=size(H,2) || nX~=size(V,1) || nX~=size(F,1) ||nX~=size(F,2))
    error('# of state variable given by the initial value does not agree matrix F and H and V');
elseif (nEps ~= size(eps_var,1) || nEps ~= size(eps_var,2))
    error('# of state shock does not agree the dimention of eps_var');
elseif (nEta ~= size(eta_var,1) || nEta ~= size(eta_var,2))
    error('# of measurement error does not agree the dimention of eta_var');    
end
%Initialise matrices:
X_pre_mean = zeros(T,nX);   %state prediction: mean 
X_pre_var = zeros(nX,nX,T); %state prediction: variance 
X_fil = zeros(T,nX);        %state filterred: mean 
X_filvar = zeros(nX,nX,T);  %state filterred: variance 
Y_pre_mean = zeros(T,nY);   %measurement prediction: forecast
Y_pre_error = zeros(T,nY);  %measurement prediction: forecast error
Y_pre_var = zeros(nY,nY,T); %measurement prediction: forecast error variance
loglik = zeros(T,1);        %record log-likelihood

%% KALMAN FILTER
for i= 1:T
    if i == 1
        %Predicted X value and variance given initial condition
        X_pre_mean(i,:) = F*InitX_mean;
        X_pre_var(:,:,i) = F*InitX_var*F' + V*eps_var*V';
        
        %Predicted Y value, forecast error and variance given initial condition
        Y_pre_mean(i,:) = H*X_pre_mean(i,:);
        Y_pre_error(i,:) = data(i,:) - Y_pre_mean(i,:);
        Y_pre_var(:,:,i) = H*X_pre_var(:,:,i)*H' + W*eta_var*W';
        
        %Update X state value and variance given the first observation of Y
        X_fil(i,:) =  X_pre_mean(i,:) +...
            X_pre_var(:,:,i)*H'*(Y_pre_var(:,:,i))^(-1)*Y_pre_error(i,:)';
        X_filvar(:,:,i) = X_pre_var(:,:,i) -...
            X_pre_var(:,:,i)*H'*(Y_pre_var(:,:,i))^(-1)*H*X_pre_var(:,:,i);
    else
        %Predicted X value and variance given t-1 information
        X_pre_mean(i,:) = F*X_fil(i-1,:);
        X_pre_var(:,:,i) = F*X_filvar(:,:,i-1)*F' + V*eps_var*V';
        
        %Predicted Y value, forecast error and variance given t-1 information
        Y_pre_mean(i,:) = H*X_pre_mean(i,:);
        Y_pre_error(i,:) = data(i,:)-Y_pre_mean(i,:);
        Y_pre_var(:,:,i) = H*X_pre_var(:,:,i)*H' + W*eta_var*W';
        
        %Update X state value and variance given the first observation of Y
        X_fil(i,:) =  X_pre_mean(i,:) +...
            X_pre_var(:,:,i)*H'*(Y_pre_var(:,:,i))^(-1)*Y_pre_error(i,:)';
        X_filvar(:,:,i) = X_pre_var(:,:,i) -...
            X_pre_var(:,:,i)*H'*(Y_pre_var(:,:,i))^(-1)*H*X_pre_var(:,:,i);
    end
    %Evaluate log-likelihood
    loglik(i,1) = -0.5*(log(2*pi) + log(det(Y_pre_var(:,:,i)))...
        + Y_pre_error(i,:)*Y_pre_var(:,:,i)*Y_pre_error(i,:)');
end


