clear all
%This file contains the model information for the methodology chapter:
% -------------------------------------------------------------------------
% The rational expectation model includes:
%  A NKPC: (pihat = pi)                          
%     pi = beta*pi(+1) + [(1-beta*theta)*(1-theta)/theta]*(y + sigmaL*n)
%
%  Dynamic IS curve:: (yhat = y)                                   
%     y = y(+1) - r
%
%  Hours worked (nhat = n)    
%     (sigmaL + alpha)*n = a - y
%
%  Technology:(ahat = a)               
%     a = rho_a*a(-1) + e_a  
%
%  Monetary Policy (rhat = r)                         
%     r = kappa_y*y + kappa_pi*pi + e_r
%
% The model can be reduced to:
%  A NKPC: (pihat = pi)                          
%     pi = beta*pi(+1) 
%        + ((1-beta*theta)*(1-theta)*sigmaL/(theta*(sigmaL + alpha)))*y
%        + ((1-beta*theta)*(1-theta)*alpha/(theta*(sigmaL + alpha)))*a
%        = beta*pi(+1) + xi_y*y + xi_a*a
%
%  Dynamic IS curve: (yhat = y) 
% (combine MP r = kappa_pi*pi + kappa_y*y + e_r with Dynamic IS: y=y(+1)-r)                                 
%     
%     (1+kappa_y)*y = y(+1) - kappa_pi*pi  - e_r
%
%  Technology:(ahat = a)               
%     a = rho_a*a(-1) + e_a  

% -------------------------------------------------------------------------

% NOTE: Chris Sim's GENSYS.m is used to solve for the rational expectation
% model, the general format for equation to be solved is:
%          R0*Y(t) = R1*Y(t-1) + C + Psi*Z(t) + Pi*Eta(t)
% Z(t) is exogenous shock vector and Eta(t) is the expectational error vec

% Author: Ben Wang
% @Macquarie University, Sydney
% File created:  09/12/2010
% Last revision: 04/03/2011 

%------------- BEGIN CODE --------------

%% Define structural parameters and set up the model
param_fix.alpha = 0.4;            %%  labour share of production                       
param_fix.sigmaL = 1;             %%  inverse of frisch elasticity                     
param_fix.beta = 0.98;            %%  time preference                      
param_fix.theta = 0.65;           %%  calvo probability                     
param_fix.rho_a = 0.9;            %%  technology persistence                        
param_fix.kappa_y  = 1.5;         %%  Taylor rule parameter on output                          
param_fix.kappa_pi = 1.2;         %%  Taylor rule parameter on inflation;                         
%VCV parameter to be estimated
param_fix.var_eps_a = 0.01^2;
param_fix.var_eps_r = 0.01^2;


X0(1,1) = param_fix.rho_a;
X0(2,1) = param_fix.var_eps_a;
X0(3,1) = param_fix.var_eps_r;

%Kalman filter parameter:
kalman_init.mean = 0;
kalman_init.variance = 0.01^2;

%Load data file and write into observation matrix VarObs
Meth_Chap_data
VarObs = [y,pi];

%% Maximum Likelihood Estimation

%Define maximum iteration
MaxIter = 1000;
%Convergence criterion. Iteration will cease when it proves impossible to 
%improve the function value by more than crit.
crit = 1e-7;
H_init = 1e-17*eye(length(X0));

[fval,param_est,grad,hessian,itct,fcount,retcodehat] = ...
                csminwel('likelihood',X0,H_init,[],crit,MaxIter,param_fix,kalman_init,VarObs);
            
disp('--------------------------------------------------------------- ')           
disp('                         RESULTS:')                  
fprintf('Estimated tech persistence is %f \n',param_est(1))
fprintf('Estimated tech shock variance is %d \n',param_est(2))
fprintf('Estimated interest rate shock variance is %d \n',param_est(3))
            
%x = fminsearch('likelihood1',X0);
%x = fminunc('likelihood1',X0);
%[x] = fminunc('likelihood1',X0);


%[fh,xh,gh,H,itct,fcount,retcodeh] =
%csminwel(fcn,x0,H0,grad,crit,nit,varargin)

%Perform Kalman filter using my kalman.m
%[X_fil, X_filvar, Y_pre_error, Y_pre_var, X_pre_var, loglik] = ...
%    Kalman(VarObs, F,V,H,W, Init_mean, Init_variance, var_eps_a, var_eps_r);










