function [F,V,H,W] =Model_Info(param_init,param_fix)
%Model_info parameterises the small NKM used in the methodology chapter
%into state space form:
%
%   State equation:        X(t) = F*X(t-1) + V*eps
%   Measurement equation:  Y(t) = H*X(t) + W*eta
%
% USAGE:
%   Model_Info takes the value of structural parameters as input argument. 
%   The file then sets up matrices and calls Sim's Gensys.m to solve the 
%   NKM. The file then re-organises the output from Gensys, puts into 
%   F,V,H,W, which are used for Kalman.m to perform Kalman filter.
% -------------------------------------------------------------------------
% RETURNS: 
%            F, V, H ,W
%
% Author: Ben Wang
% @Macquarie University, Sydney
% File created:  04/03/2011
% Last revision: 04/03/2011
%------------- BEGIN CODE --------------

%% INITIALISE PARAMETERS:
%All calibrated values are given in section 6 on page 26-27

alpha = param_fix.alpha;                  %%  labour share of production
sigmaL = param_fix.sigmaL;                %%  inverse of frisch elasticity
beta = param_fix.beta;                    %%  time preference
theta = param_fix.theta;                  %%  calvo probability
rho_a = param_init.rho_a;                  %%  technology persistence
kappa_y = param_fix.kappa_y;              %%  Taylor rule parameter on output
kappa_pi = param_fix.kappa_pi;            %%  Taylor rule parameter on infl
%Reduced parameters for y and a in the NKPC:
xi_a = ((1-beta*theta)*(1-theta)*sigmaL/(theta*(sigmaL + alpha)));
xi_y = ((1-beta*theta)*(1-theta)*alpha/(theta*(sigmaL + alpha)));
%% DEFINE MODEL
%Model:    R0*Y(t) = R1*Y(t-1) + C + Psi*Z(t) + Pi*Eta(t)
%NOTE: Y(t) = [ a(t)
%               y(t+1)
%               y(t)
%               pi(t+1)
%               pi(t)];

%Equation order:  (1) Technology evolution
%                 (2) Dynamic IS curve
%                 (3) Expectation error equation for output y
%                 (4) NKPC
%                 (5) Expectation error equation for inflation pi
R0 = [   1,  0,            0,    0,         0;
         0,  1, -(1+kappa_y),    0, -kappa_pi;
         0,  0,            1,    0,         0;
      xi_a,  0,         xi_y, beta,        -1;
         0,  0,            0,    0,         1];

R1 = [rho_a, 0, 0, 0, 0;
          0, 0, 0, 0, 0;
          0, 1, 0, 0, 0;
          0, 0, 0, 0, 0;
          0, 0, 0, 1, 0];  
      
c = [0; 0; 0; 0; 0];

Psi = [1, 0;
       0, 1;
       0, 0;
       0, 0;
       0, 0];
   
Pi = [0, 0;
      0, 0;
      1, 0;
      0, 0;
      0, 1];
 %% SOVE MODEL BY USING GENSYS.M BY SIMS
%Call Sim's file to solve the system:
%The solution is given by:     Y(t)=G1*Y(t-1)+C+impact*z(t)
[G1,~,impact] = gensys(R0,R1,c,Psi,Pi);
%% TRANSFORM GENSYS OUTPUT
%The output produced by gensys.m is in the following way:
%   [     a_t    ]           [    a_t-1    ]
%   |  Et[y_t+1] |           |  Et-1[y_t]  |
%   |     y_t    |  =  G1*   |    y_t-1    | 
%   | Et[pi_t+1] |           | Et-1[pi_t]  |
%   [    pi_t    ]           [   pi_t-1    ]
%Since we are not interested the elements in G1 describing how Et-1[*]
%evolves, so we need to get rid of the 2ed and 4th columns and rows,
%which reduce G1 into a 3x3 matrix. We process impact matrix by the same 
%procedure, that deletes the 2ed and 4th rows gives a 3x2 matrix. The 
%reduced solution is: Y(t)=RG1*Y(t-1) + Rimpact*z(t), where
%   [     a_t    ]               
%   |     y_t    |  =  Y(t)
%   [    pi_t    ]         
t = 1;
tt= 1;
%Perform row reduction
for i = 1:5
    if i == 2
        t = t;
    elseif i == 4
        t = t;
    else
        RG1_aux(t,:) = G1(i,:);
        Rimpact(t,:) = impact(i,:);
        t = t+1;
    end
end  
%Perform column reduction for G1
for ii = 1:5
    if ii == 2
        tt = tt;
    elseif ii == 4
        tt = tt;
    else
        RG1(:,tt) = RG1_aux(:,ii);
        tt = tt+1;
    end
end
%% Re-organise the Gensys.m output, and put into the state space form:

%The reduced RG1 and Rimpact can be decomposed into the state space form
%for evaluating the likelihood by Kalman Filter. Define the state space
%form as in:
%
%   State equation:        X(t) = F*X(t-1) + V*eps(t)
%   Measurement equation:  Y(t) = H*X(t) + W*eta(t)
%
%Since the model after transforming from last section reduces to:
%   [     a_t    ]           [    a_t-1    ]
%   [     y_t    ]  =  RG1*  [    y_t-1    ] + Rimpact* [ eps_a ]
%   [    pi_t    ]           [   pi_t-1    ]            [ eps_r ]  
%and we need y_t and pi_t depends on a_t not a_t-1, therefor we need to
%divide F from RG1 in order to get H. 

%Setup F, H and W:
F = RG1(1,1);
H = RG1(2:3,1)./F;
W = Rimpact(2:3,2);
V = 1;
end
